Package: salesforcer
Title: An Implementation of 'Salesforce' APIs Using Tidy Principles
Version: 1.0.2
Date: 2024-11-05
Description: Functions connecting to the 'Salesforce' Platform APIs (REST, SOAP, 
    Bulk 1.0, Bulk 2.0, Metadata, Reports and Dashboards) 
    <https://trailhead.salesforce.com/content/learn/modules/api_basics/api_basics_overview>. 
    "API" is an acronym for "application programming interface". Most all calls 
    from these APIs are supported as they use CSV, XML or JSON data that can be 
    parsed into R data structures. For more details please see the 'Salesforce' 
    API documentation and this package's website 
    <https://stevenmmortimer.github.io/salesforcer/> for more information, 
    documentation, and examples.
Authors@R: 
    c(person(given = c("Steven", "M."),
             family = "Mortimer",
             role = c("aut", "cre"),
             email = "mortimer.steven.m@gmail.com"),
      person(given = "Takekatsu",
             family = "Hiramura",
             role = "ctb",
             email = "thira@plavox.info"),
      person(given = "Jennifer",
             family = "Bryan",
             role = c("ctb", "cph"),
             email = "jenny@rstudio.com"),
      person(given = "Joanna",
             family = "Zhao",
             role = c("ctb", "cph"),
             email = "joanna.zhao@alumni.ubc.ca"))
License: MIT + file LICENSE
URL: https://github.com/StevenMMortimer/salesforcer,
        https://stevenmmortimer.github.io/salesforcer/
BugReports: https://github.com/StevenMMortimer/salesforcer/issues
Depends: R (>= 3.6.0)
Imports: methods (>= 3.6.0), utils (>= 3.6.0), stats (>= 3.6.0), dplyr
        (>= 1.0.0), purrr (>= 0.3.4), vctrs (>= 0.3.4), tibble (>=
        3.0.3), readr (>= 1.3.1), lubridate (>= 1.7.8), anytime (>=
        0.3.9), rlang (>= 0.4.7), httr (>= 1.4.1), curl (>= 4.3),
        data.table (>= 1.13.0), XML (>= 3.99-0.3), xml2 (>= 1.3.2),
        jsonlite (>= 1.6.1), rlist (>= 0.4.6.1), zip (>= 2.0.4),
        base64enc (>= 0.1-3), mime (>= 0.9), lifecycle (>= 0.2.0)
Suggests: knitr, rmarkdown, testthat, spelling, here, microbenchmark,
        ggplot2, sessioninfo
VignetteBuilder: knitr
ByteCompile: true
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2024-11-05 20:57:23 UTC; steven.mortimer
Author: Steven M. Mortimer [aut, cre],
  Takekatsu Hiramura [ctb],
  Jennifer Bryan [ctb, cph],
  Joanna Zhao [ctb, cph]
Maintainer: Steven M. Mortimer <mortimer.steven.m@gmail.com>
Repository: CRAN
Date/Publication: 2024-11-06 10:00:01 UTC
Built: R 4.6.0; ; 2026-04-26 19:02:22 UTC; unix
