# Generated by roxygen2: do not edit by hand

S3method(sf_format_time,"NULL")
S3method(sf_format_time,AsIs)
S3method(sf_format_time,Date)
S3method(sf_format_time,POSIXct)
S3method(sf_format_time,POSIXlt)
S3method(sf_format_time,POSIXt)
S3method(sf_format_time,character)
S3method(sf_format_time,data.frame)
S3method(sf_format_time,list)
S3method(sf_format_time,logical)
S3method(sf_format_time,numeric)
export(VERB_n)
export(accepted_controls_by_api)
export(accepted_controls_by_operation)
export(bind_query_resultsets)
export(build_manifest_xml_from_list)
export(build_metadata_xml_from_list)
export(build_proxy)
export(build_soap_xml_from_list)
export(catch_errors)
export(catch_unknown_api)
export(check_and_encode_files)
export(collapse_list_with_dupe_names)
export(combine_parent_and_child_resultsets)
export(compact2)
export(drop_attributes)
export(drop_attributes_recursively)
export(drop_empty_recursively)
export(drop_nested_child_records)
export(extract_nested_child_records)
export(extract_records_from_xml_node)
export(extract_records_from_xml_nodeset)
export(extract_records_from_xml_nodeset_of_records)
export(filter_valid_controls)
export(flatten_tbl_df)
export(format_headers_for_verbose)
export(format_report_row)
export(get_os)
export(guess_object_name_from_soql)
export(is_legit_token)
export(list_extract_parent_and_child_result)
export(make_analytics_folder_child_operations_url)
export(make_analytics_folder_collections_url)
export(make_analytics_folder_operations_url)
export(make_analytics_folder_share_by_id_url)
export(make_analytics_folder_share_recipients_url)
export(make_analytics_folder_shares_url)
export(make_analytics_notification_operations_url)
export(make_analytics_notifications_limits_url)
export(make_analytics_notifications_list_url)
export(make_base_metadata_url)
export(make_base_rest_url)
export(make_base_soap_url)
export(make_bulk_batch_details_url)
export(make_bulk_batch_status_url)
export(make_bulk_batches_url)
export(make_bulk_create_job_url)
export(make_bulk_delete_job_url)
export(make_bulk_end_job_generic_url)
export(make_bulk_get_all_jobs_url)
export(make_bulk_get_all_query_jobs_url)
export(make_bulk_get_job_url)
export(make_bulk_job_records_url)
export(make_bulk_query_result_url)
export(make_bulk_query_url)
export(make_chatter_users_url)
export(make_composite_batch_url)
export(make_composite_url)
export(make_dashboard_copy_url)
export(make_dashboard_describe_url)
export(make_dashboard_filter_operators_list_url)
export(make_dashboard_filter_options_analysis_url)
export(make_dashboard_status_url)
export(make_dashboard_url)
export(make_dashboards_list_url)
export(make_login_url)
export(make_parameterized_search_url)
export(make_query_url)
export(make_report_copy_url)
export(make_report_create_url)
export(make_report_describe_url)
export(make_report_execute_url)
export(make_report_fields_url)
export(make_report_filter_operators_list_url)
export(make_report_instance_url)
export(make_report_instances_list_url)
export(make_report_query_url)
export(make_report_type_describe_url)
export(make_report_types_list_url)
export(make_report_url)
export(make_reports_list_url)
export(make_rest_describe_url)
export(make_rest_objects_url)
export(make_rest_record_url)
export(make_search_url)
export(make_soap_xml_skeleton)
export(make_verbose_httr_message)
export(map_sf_type_to_r_type)
export(merge_null_to_na)
export(message_w_errors_listed)
export(metadata_type_validator)
export(parameterized_search_control)
export(parse_error_code_and_message)
export(parse_report_detail_rows)
export(patched_tempdir)
export(rDELETE)
export(rGET)
export(rPATCH)
export(rPOST)
export(rPUT)
export(records_list_to_tbl)
export(remove_empty_linked_object_cols)
export(return_matching_controls)
export(rforcecom.bulkAction)
export(rforcecom.bulkQuery)
export(rforcecom.create)
export(rforcecom.delete)
export(rforcecom.getObjectDescription)
export(rforcecom.getServerTimestamp)
export(rforcecom.login)
export(rforcecom.query)
export(rforcecom.retrieve)
export(rforcecom.search)
export(rforcecom.update)
export(rforcecom.upsert)
export(safe_bind_rows)
export(salesforcer_state)
export(session_id_available)
export(set_null_elements_to_na)
export(set_null_elements_to_na_recursively)
export(sf_abort_job_bulk)
export(sf_access_token)
export(sf_analytics_notification_create)
export(sf_analytics_notification_delete)
export(sf_analytics_notification_describe)
export(sf_analytics_notification_update)
export(sf_analytics_notifications_limits)
export(sf_analytics_notifications_list)
export(sf_auth)
export(sf_auth_check)
export(sf_auth_refresh)
export(sf_batch_details_bulk)
export(sf_batch_status_bulk)
export(sf_build_cols_spec)
export(sf_bulk_operation)
export(sf_close_job_bulk)
export(sf_control)
export(sf_convert_lead)
export(sf_copy_dashboard)
export(sf_copy_report)
export(sf_create)
export(sf_create_attachment)
export(sf_create_batches_bulk)
export(sf_create_job_bulk)
export(sf_create_job_bulk_v1)
export(sf_create_job_bulk_v2)
export(sf_create_metadata)
export(sf_create_report)
export(sf_delete)
export(sf_delete_attachment)
export(sf_delete_dashboard)
export(sf_delete_job_bulk)
export(sf_delete_metadata)
export(sf_delete_report)
export(sf_delete_report_instance)
export(sf_describe_dashboard)
export(sf_describe_dashboard_components)
export(sf_describe_metadata)
export(sf_describe_object_fields)
export(sf_describe_objects)
export(sf_describe_report)
export(sf_describe_report_type)
export(sf_download_attachment)
export(sf_empty_recycle_bin)
export(sf_end_job_bulk)
export(sf_execute_report)
export(sf_filter_dashboard_operators_list)
export(sf_filter_dashboard_options_analysis)
export(sf_find_duplicates)
export(sf_find_duplicates_by_id)
export(sf_format_date)
export(sf_format_datetime)
export(sf_format_time)
export(sf_get_all_jobs_bulk)
export(sf_get_all_query_jobs_bulk)
export(sf_get_dashboard_data)
export(sf_get_dashboard_results)
export(sf_get_dashboard_status)
export(sf_get_deleted)
export(sf_get_job_bulk)
export(sf_get_job_records_bulk)
export(sf_get_report_instance_results)
export(sf_get_updated)
export(sf_guess_cols)
export(sf_input_data_validation)
export(sf_job_batches_bulk)
export(sf_list_api_limits)
export(sf_list_dashboards)
export(sf_list_metadata)
export(sf_list_objects)
export(sf_list_report_fields)
export(sf_list_report_filter_operators)
export(sf_list_report_instances)
export(sf_list_report_types)
export(sf_list_reports)
export(sf_list_resources)
export(sf_list_rest_api_versions)
export(sf_merge)
export(sf_query)
export(sf_query_bulk)
export(sf_query_bulk_v1)
export(sf_query_bulk_v2)
export(sf_query_report)
export(sf_query_result_bulk)
export(sf_query_result_bulk_v1)
export(sf_query_result_bulk_v2)
export(sf_read_metadata)
export(sf_refresh_dashboard)
export(sf_rename_metadata)
export(sf_reorder_cols)
export(sf_report_folder_children)
export(sf_report_folder_create)
export(sf_report_folder_delete)
export(sf_report_folder_describe)
export(sf_report_folder_share_delete)
export(sf_report_folder_share_describe)
export(sf_report_folder_share_recipients)
export(sf_report_folder_share_update)
export(sf_report_folder_shares_add)
export(sf_report_folder_shares_list)
export(sf_report_folder_shares_update)
export(sf_report_folder_update)
export(sf_report_folders_list)
export(sf_reset_password)
export(sf_rest_list)
export(sf_retrieve)
export(sf_retrieve_metadata)
export(sf_retrieve_metadata_check_status)
export(sf_run_bulk_operation)
export(sf_run_bulk_query)
export(sf_run_report)
export(sf_search)
export(sf_server_timestamp)
export(sf_session_id)
export(sf_set_dashboard_sticky_filter)
export(sf_set_password)
export(sf_submit_query_bulk)
export(sf_undelete)
export(sf_update)
export(sf_update_attachment)
export(sf_update_dashboard)
export(sf_update_metadata)
export(sf_update_report)
export(sf_upload_complete_bulk)
export(sf_upsert)
export(sf_upsert_metadata)
export(sf_user_info)
export(sf_write_csv)
export(simplify_report_metadata)
export(stop_w_errors_listed)
export(token_available)
export(unbox_list_elements)
export(unbox_list_elements_recursively)
export(unnest_col)
export(valid_metadata_list)
export(validate_get_all_jobs_params)
export(warn_w_errors_listed)
export(xmlToList2)
export(xml_drop_and_unlist)
export(xml_drop_and_unlist_recursively)
export(xml_extract_parent_and_child_result)
export(xml_nodeset_to_df)
importFrom(XML,"xmlValue<-")
importFrom(XML,addChildren)
importFrom(XML,newXMLNode)
importFrom(XML,saveXML)
importFrom(XML,xmlApply)
importFrom(XML,xmlAttrs)
importFrom(XML,xmlChildren)
importFrom(XML,xmlDoc)
importFrom(XML,xmlInternalTreeParse)
importFrom(XML,xmlParse)
importFrom(XML,xmlRoot)
importFrom(XML,xmlSApply)
importFrom(XML,xmlSize)
importFrom(XML,xmlToList)
importFrom(XML,xmlValue)
importFrom(anytime,anydate)
importFrom(anytime,anytime)
importFrom(base64enc,base64encode)
importFrom(curl,form_data)
importFrom(curl,form_file)
importFrom(data.table,rbindlist)
importFrom(dplyr,"%>%")
importFrom(dplyr,across)
importFrom(dplyr,all_of)
importFrom(dplyr,any_of)
importFrom(dplyr,as_tibble)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,contains)
importFrom(dplyr,everything)
importFrom(dplyr,expr)
importFrom(dplyr,filter)
importFrom(dplyr,if_all)
importFrom(dplyr,is.tbl)
importFrom(dplyr,matches)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,mutate_if)
importFrom(dplyr,one_of)
importFrom(dplyr,rename)
importFrom(dplyr,rename_at)
importFrom(dplyr,rename_with)
importFrom(dplyr,select)
importFrom(dplyr,tibble)
importFrom(httr,DELETE)
importFrom(httr,GET)
importFrom(httr,PATCH)
importFrom(httr,POST)
importFrom(httr,PUT)
importFrom(httr,RETRY)
importFrom(httr,add_headers)
importFrom(httr,build_url)
importFrom(httr,config)
importFrom(httr,content)
importFrom(httr,headers)
importFrom(httr,http_error)
importFrom(httr,oauth2.0_token)
importFrom(httr,oauth_app)
importFrom(httr,oauth_endpoint)
importFrom(httr,parse_url)
importFrom(httr,status_code)
importFrom(httr,upload_file)
importFrom(httr,use_proxy)
importFrom(jsonlite,base64_dec)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,prettify)
importFrom(jsonlite,toJSON)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecate_warn)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(lubridate,as_datetime)
importFrom(lubridate,dmy_hms)
importFrom(lubridate,is.Date)
importFrom(lubridate,is.POSIXct)
importFrom(lubridate,is.POSIXlt)
importFrom(lubridate,is.POSIXt)
importFrom(methods,as)
importFrom(mime,guess_type)
importFrom(purrr,as_mapper)
importFrom(purrr,compact)
importFrom(purrr,discard)
importFrom(purrr,list_modify)
importFrom(purrr,map)
importFrom(purrr,map_chr)
importFrom(purrr,map_depth)
importFrom(purrr,map_df)
importFrom(purrr,map_dfc)
importFrom(purrr,map_if)
importFrom(purrr,map_lgl)
importFrom(purrr,modify)
importFrom(purrr,modify_at)
importFrom(purrr,modify_if)
importFrom(purrr,pluck)
importFrom(purrr,set_names)
importFrom(purrr,transpose)
importFrom(readr,col_character)
importFrom(readr,col_guess)
importFrom(readr,cols)
importFrom(readr,locale)
importFrom(readr,parse_datetime)
importFrom(readr,read_csv)
importFrom(readr,type_convert)
importFrom(readr,write_csv)
importFrom(rlang,is_empty)
importFrom(rlist,list.flatten)
importFrom(stats,quantile)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(tibble,as_tibble_row)
importFrom(tools,file_ext)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,object.size)
importFrom(utils,tail)
importFrom(vctrs,vec_as_names)
importFrom(xml2,as_list)
importFrom(xml2,read_xml)
importFrom(xml2,url_escape)
importFrom(xml2,xml_add_child)
importFrom(xml2,xml_add_sibling)
importFrom(xml2,xml_child)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_new_document)
importFrom(xml2,xml_ns_strip)
importFrom(xml2,xml_remove)
importFrom(xml2,xml_set_namespace)
importFrom(xml2,xml_text)
importFrom(zip,zipr)
